@@ -10,6 +10,7 @@
 
 import datetime
 import logging
+import sys
 from collections import defaultdict
 
 from . import __version__
@@ -196,7 +197,7 @@ class EPUBased(Coster):
 
         return result
 
-    def summary(self, title):
+    def summary(self, title, max_charge_limit=None):
         super().summary(title)
         print("   EPUs:            ", "{:,}".format(int(self.epus)))
         print("   Discounted EPUs: ", "{:,}".format(int(self.discounted_epus)))
@@ -209,6 +210,10 @@ class EPUBased(Coster):
             print(f" ({result})")
         else:
             print()
+        if max_charge_limit is not None:
+            max_charge_order = self.euros >= max_charge_limit
+            print(f"   Max charge limit: EUR {max_charge_limit:,}")
+            print(f"   Max charge order: {'yes' if max_charge_order else 'no'}")
         print()
 
     def update_subcosts(self, costs):
@@ -325,8 +330,24 @@ class Costing:
 
         result["total"] = self.totals.as_dict()
 
+        result["max_charge_limit"] = self.max_charge_limit
+        result["max_charge"] = self.totals.euros >= result["max_charge_limit"]
+
         return result
 
+    @property
+    def chargeable_high_frequency(self):
+        hf = self.per_group.get("high-frequency")
+        if hf is None:
+            return False
+        return hf.euros > 0
+
+    @property
+    def max_charge_limit(self):
+        if self.chargeable_high_frequency:
+            return config("max-charge-high-frequency")
+        return config("max-charge-core")
+
     def summary(
         self,
         *,
@@ -501,7 +522,42 @@ class Costing:
 
         self.totals.update_subcosts(self.per_group)
 
-        self.totals.summary("Grand total:")
+        self.totals.summary("Grand total:", self.max_charge_limit)
+
+    def csv(
+        self,
+        file=sys.stdout,
+        groups=False,
+        targets=False,
+        categories=False,
+        users=False,
+    ):
+        import csv
+
+        fieldnames = []
+        rows = []
+
+        if categories:
+            for category, coster in sorted(self.per_category.items()):
+                rows.append(dict(category=category, **coster.as_dict()))
+
+        if users:
+            for user, coster in sorted(self.per_user.items()):
+                rows.append(dict(user=user, **coster.as_dict()))
+
+        if targets:
+            for target, coster in sorted(self.per_target.items()):
+                rows.append(dict(target=target, **coster.as_dict()))
+
+        if groups:
+            for group, coster in sorted(self.per_group.items()):
+                rows.append(dict(group=group, **coster.as_dict()))
+
+        fieldnames = list(rows[0].keys())
+
+        writer = csv.DictWriter(file, fieldnames=fieldnames)
+        writer.writeheader()
+        writer.writerows(rows)
 
 
 def costing(requests):
